# Generated by Django 3.2.7 on 2021-09-13 09:41

from django.db import migrations, models
import django.db.models.deletion
import pathlib


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Attendee',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=256, verbose_name='Prénom')),
                ('last_name', models.CharField(max_length=256, verbose_name='Nom')),
                ('dob', models.DateField(verbose_name='Date de naissance')),
                ('status', models.SmallIntegerField(choices=[(-1, 'Abandonné'), (0, 'Inscrit'), (1, 'Non sélectionné'), (2, 'Accepté'), (3, 'Confirmé')], verbose_name='Statut de la candidature')),
            ],
            options={
                'verbose_name': 'participant',
                'verbose_name_plural': 'participants',
            },
        ),
        migrations.CreateModel(
            name='AttendeeLabel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=120, verbose_name='Titre')),
            ],
            options={
                'verbose_name': 'label participant',
                'verbose_name_plural': 'labels participant',
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='Center',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=120, verbose_name='Nom')),
                ('address', models.TextField(max_length=1000, verbose_name='Adresse')),
                ('lng', models.DecimalField(blank=True, decimal_places=6, max_digits=9, null=True, verbose_name='Longitude')),
                ('lat', models.DecimalField(blank=True, decimal_places=6, max_digits=9, null=True, verbose_name='Latitude')),
                ('private_notes', models.TextField(blank=True, max_length=2000, null=True, verbose_name='Notes privées')),
            ],
            options={
                'verbose_name': 'centre',
                'verbose_name_plural': 'centres',
            },
        ),
        migrations.CreateModel(
            name='Document',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('admin_name', models.CharField(help_text="Ce nom est uniquement utilisé pour l'admin django", max_length=260, unique=True, verbose_name='Nom du document')),
                ('file', models.FileField(upload_to=pathlib.PurePosixPath('eventsd/documents'), verbose_name='Fichier')),
            ],
            options={
                'verbose_name': 'document',
                'verbose_name_plural': 'documents',
            },
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=256, verbose_name="Nom de l'évènement")),
                ('signup_start_date', models.DateTimeField(verbose_name="Date de début d'inscription")),
                ('signup_end_date', models.DateTimeField(verbose_name="Date de fin d'inscription")),
                ('start_date', models.DateTimeField(verbose_name='Date de début')),
                ('end_date', models.DateTimeField(verbose_name='Date de fin')),
                ('notes', models.TextField(blank=True, help_text="Ce texte, si défini est affiché lors de l'inscription à l'évènement (il peut éventuellement contenir des liens)", null=True, verbose_name='Notes')),
                ('description', models.TextField(help_text="Si ce champ n'est pas spécifié, la description du projet sera utilisée.", verbose_name="Description de l'évènement")),
                ('center', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eventsd_events.center', verbose_name='Centre')),
            ],
            options={
                'verbose_name': 'évènement',
                'verbose_name_plural': 'évènements',
            },
        ),
        migrations.CreateModel(
            name='Form',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=120, verbose_name='Nom')),
            ],
            options={
                'verbose_name': 'formulaire',
                'verbose_name_plural': 'formulaires',
            },
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.CharField(max_length=1000, verbose_name='Texte de la question')),
                ('type', models.CharField(choices=[('TITLE', 'Titre'), ('TEXT', 'Texte'), ('LONG_TEXT', 'Texte long'), ('CHOICE', 'Choix'), ('MULTIPLE_CHOICES', 'Choix multiple')], max_length=32, verbose_name='Type')),
                ('mandatory', models.BooleanField(verbose_name='Obligatoire')),
                ('answers', models.TextField(blank=True, help_text='Une réponse par ligne', max_length=500, null=True, verbose_name='Réponses possibles')),
                ('order', models.PositiveSmallIntegerField(verbose_name='Ordre')),
                ('form', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='questions', to='eventsd_events.form', verbose_name='Formulaire')),
            ],
            options={
                'verbose_name': 'question',
                'verbose_name_plural': 'questions',
                'ordering': ('order',),
            },
        ),
        migrations.CreateModel(
            name='FormAnswer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('answer', models.TextField(max_length=10000, verbose_name='Réponse')),
                ('attendee', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='form_answers', to='eventsd_events.attendee', verbose_name='Participant')),
                ('question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='form_answers', to='eventsd_events.question', verbose_name='Question')),
            ],
            options={
                'verbose_name': 'réponse formulaire',
                'verbose_name_plural': 'réponses formulaire',
                'ordering': ('question__order',),
            },
        ),
        migrations.CreateModel(
            name='EventDocument',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('display_name', models.CharField(help_text='Le nom public du document qui sera affiché aux participants', max_length=260, verbose_name='Nom public du document')),
                ('visibility', models.IntegerField(choices=[(1, 'Public'), (2, 'Accepté ou confirmé'), (3, 'Confirmé seulement')], verbose_name='Visibilité')),
                ('document', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eventsd_events.document', verbose_name='Document')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eventsd_events.event', verbose_name='Évènement')),
            ],
            options={
                'verbose_name': "document lié à l'évènement",
                'verbose_name_plural': "documents lié à l'évènement",
            },
        ),
        migrations.AddField(
            model_name='event',
            name='documents',
            field=models.ManyToManyField(blank=True, related_name='events', through='eventsd_events.EventDocument', to='eventsd_events.Document', verbose_name='Documents'),
        ),
        migrations.AddField(
            model_name='event',
            name='form',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.SET_DEFAULT, to='eventsd_events.form', verbose_name='Formulaire'),
        ),
        migrations.AddField(
            model_name='attendee',
            name='event',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eventsd_events.event', verbose_name='Évènement'),
        ),
        migrations.AddField(
            model_name='attendee',
            name='labels',
            field=models.ManyToManyField(blank=True, to='eventsd_events.AttendeeLabel', verbose_name='Labels'),
        ),
    ]



# Generated by Django 3.2.7 on 2021-09-22 18:29

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, unique=True, verbose_name='ID')),
                ('email', models.EmailField(max_length=254, verbose_name='Adresse électronique')),
                ('confirmed', models.BooleanField(default=False, verbose_name='Confirmé')),
            ],
        ),
    ]



from rest_framework import (
    viewsets,
    permissions,
)

from .. import (
    models,
    serializers,
)


class FormViewset(viewsets.ReadOnlyModelViewSet):
    permission_classes = [permissions.IsAuthenticated]
    queryset = models.Form.objects.all()
    serializer_class = serializers.FormSerializer


class AttendeeViewset(viewsets.ModelViewSet):
    permission_classes = [permissions.IsAuthenticated]
    serializer_class = serializers.AttendeeSerializer
    http_method_names = ["get", "post", "delete"]

    def get_queryset(self):
        return models.Attendee.objects.filter(
            owner=self.request.user,
        )



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class NewsletterConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = "eventsd.newsletter"
    label = "eventsd_newsletter"
    verbose_name = _("Newsletter")



from django.urls import path, include
from . import views
from rest_framework import routers

router = routers.SimpleRouter()

router.register("events", views.EventViewset, basename="events")
router.register("forms", views.FormViewset, basename="forms")
router.register("attendees", views.AttendeeViewset, basename="attendees")

app_name = "eventsd_events"

urlpatterns = [
    path("", include(router.urls)),
]



from django.contrib import admin
from . import models


@admin.register(models.Sponsor)
class SponsorAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "enabled",
        "featured",
        "order",
        "website_url",
    )

    list_filter = ("enabled",)



# Generated by Django 3.2.7 on 2021-09-13 09:41

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Sponsor',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=256, verbose_name='Nom')),
                ('logo', models.FileField(upload_to='', verbose_name='Logo')),
                ('enabled', models.BooleanField(verbose_name='Activé')),
                ('order', models.PositiveIntegerField(default=1, verbose_name='Ordre')),
            ],
            options={
                'ordering': ('order',),
            },
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class UsersConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    label = "eventsd_users"
    name = "eventsd.users"
    verbose_name = _("Utilisateurs")



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as UpstreamUserAdmin
from django.utils.translation import ugettext_lazy as _
from . import models


@admin.register(models.User)
class UserAdmin(UpstreamUserAdmin):
    fieldsets = (
        (None, {"fields": ("username", "password")}),
        (
            _("Informations personnelles"),
            {"fields": ("first_name", "last_name", "email")},
        ),
        (
            _("Adresse postale"),
            {"fields": ("address", "city", "zip_code", "country")},
        ),
        (
            _("Permissions"),
            {
                "fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                    "groups",
                    "user_permissions",
                ),
            },
        ),
        (_("Dates importantes"), {"fields": ("last_login", "date_joined")}),
    )

    add_fieldsets = (
        (
            None,
            {
                "classes": ("wide",),
                "fields": ("username", "password1", "password2"),
            },
        ),
    )



from rest_framework import (
    serializers,
)
from django.contrib.auth import get_user_model
from . import models


class CenterSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.Center
        fields = (
            "id",
            "name",
            "address",
            "lng",
            "lat",
        )


class EventSerializer(serializers.ModelSerializer):

    center = CenterSerializer()

    class Meta:
        depth = 1
        model = models.Event
        fields = (
            "id",
            "name",
            "center",
            "signup_start_date",
            "signup_end_date",
            "start_date",
            "end_date",
            "form_id",
            "description",
            "notes",
            "documents",
        )


class QuestionSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.Question
        fields = (
            "id",
            "order",
            "text",
            "type",
            "mandatory",
            "possible_answers",
        )


class FormSerializer(serializers.ModelSerializer):

    questions = QuestionSerializer(many=True)

    class Meta:
        model = models.Form
        fields = (
            "id",
            "questions",
        )


class FormAnswerSerializer(serializers.ModelSerializer):
    question = QuestionSerializer()

    class Meta:
        model = models.FormAnswer
        fields = (
            "id",
            "question",
            "answer",
        )

        read_only_fields = ("id",)


class FormAnswerPushSerializer(serializers.ModelSerializer):
    question = serializers.PrimaryKeyRelatedField(
        queryset=models.Question.objects.all()
    )

    class Meta:
        model = models.FormAnswer
        fields = (
            "id",
            "question",
            "answer",
        )
        write_only_fields = ("attendee_id",)
        read_only_fields = ("id",)


class AttendeeSerializer(serializers.ModelSerializer):
    owner = serializers.PrimaryKeyRelatedField(
        allow_null=True,
        queryset=get_user_model().objects.all(),
    )

    status = serializers.ChoiceField(
        choices=models.SelectionStatus.choices,
        allow_null=True,
    )

    form_answers = FormAnswerPushSerializer(many=True)

    def create(self, validated_data):
        request = self.context["request"]
        user = request.user
        validated_data["owner"] = user
        validated_data["status"] = models.SelectionStatus.ENROLLED.value
        form_answers = validated_data.pop("form_answers")
        ret = super().create(validated_data)
        for answer in form_answers:
            answer["attendee_id"] = ret.id
            FormAnswerPushSerializer().create(answer)
        return ret

    class Meta:
        model = models.Attendee
        fields = (
            "id",
            "owner",
            "event",
            "first_name",
            "last_name",
            "dob",
            "status",
            "form_answers",
        )

        read_only_fields = ("id",)



from rest_framework import views, response

class SubscribeView(views.APIView):
    http_method_names = ('post', )

    def post(self, request, *args, **kwargs):
        print(kwargs)
        return response.Response({"ok": True})


from django.urls import path, include
from rest_framework import routers
from . import views

app_name = "eventsd_sponsors"

router = routers.SimpleRouter()
router.register("", views.SponsorViewset, basename="sponsors")

urlpatterns = [
    path("", include(router.urls)),
]



from rest_framework import (
    generics,
    permissions,
)
from . import serializers


class UserMeView(generics.RetrieveUpdateAPIView):
    serializer_class = serializers.UserSerializer
    permission_classes = [
        permissions.IsAuthenticated,
    ]

    def get_object(self):
        return self.request.user



from .events import (
    Center,
    Event,
    DocumentType,
    EventDocument,
    Document,
)

from .signup import (
    SelectionStatus,
    Attendee,
    Form,
    QuestionType,
    Question,
    FormAnswer,
    AttendeeLabel,
)

__all__ = (
    "Project",
    "Center",
    "Event",
    "Document",
    "DocumentType",
    "EventDocument",
    "SelectionStatus",
    "Attendee",
    "Form",
    "QuestionType",
    "Question",
    "FormAnswer",
    "AttendeeLabel",
)



from rest_framework import serializers
from . import models


class SponsorSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.Sponsor
        fields = (
            "name",
            "description",
            "website_url",
            "featured",
            "logo",
            "order",
        )



from django import forms
from django.utils.translation import ugettext_lazy as _


class EventSignupForm(forms.Form):
    def __init__(self, *args, **kwargs):
        self.form_object = kwargs.pop("form_object")  # noqa
        super().__init__(*args, **kwargs)
        first_name = forms.fields.CharField(
            label=_("Prénom (*)"),
            max_length=256,
            label_suffix="",
            required=True,
        )

        last_name = forms.fields.CharField(
            label=_("Nom (*)"),
            max_length=256,
            label_suffix="",
            required=True,
        )

        dob = forms.fields.DateField(
            label=_("Date de naisssance (*)"),
            required=True,
        )

        initial_fields = {
            "first_name": first_name,
            "last_name": last_name,
            "dob": dob,
        }
        self.fields = initial_fields | self.form_object.get_form_fields()



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
from django.core.validators import ValidationError
from django import forms


class SelectionStatus(models.IntegerChoices):
    ABANDONED = -1, _("Abandonné")
    ENROLLED = 0, _("Inscrit")
    NOT_SELECTED = 1, _("Non sélectionné")
    ACCEPTED = 2, _("Accepté")
    CONFIRMED = 3, _("Confirmé")


class Attendee(models.Model):
    owner = models.ForeignKey(
        to=get_user_model(),
        verbose_name=_("Utilisateur"),
        on_delete=models.CASCADE,
    )

    first_name = models.CharField(
        max_length=256,
        verbose_name=_("Prénom"),
    )

    last_name = models.CharField(
        max_length=256,
        verbose_name=_("Nom"),
    )

    dob = models.DateField(
        verbose_name=_("Date de naissance"),
    )

    event = models.ForeignKey(
        to="eventsd_events.Event",
        verbose_name=_("Évènement"),
        on_delete=models.CASCADE,
    )

    status = models.SmallIntegerField(
        choices=SelectionStatus.choices,
        verbose_name=_("Statut de la candidature"),
    )

    labels = models.ManyToManyField(
        to="eventsd_events.AttendeeLabel",
        blank=True,
        verbose_name=_("Labels"),
    )

    class Meta:
        verbose_name = _("participant")
        verbose_name_plural = _("participants")

    def __str__(self):
        return f"{self.first_name} {self.last_name}@{self.event}"


class Form(models.Model):
    name = models.CharField(verbose_name=_("Nom"), max_length=120)

    class Meta:
        verbose_name = _("formulaire")
        verbose_name_plural = _("formulaires")

    def __str__(self):
        return self.name

    def get_form_fields(self):
        return {
            f[0]: f[1]
            for f in [q.get_field_tuple() for q in self.questions.all()]
        }


class QuestionType(models.TextChoices):
    TITLE = "TITLE", _("Titre")
    TEXT = "TEXT", _("Texte")
    LONG_TEXT = "LONG_TEXT", _("Texte long")
    CHOICE = "CHOICE", _("Choix")
    MULTIPLE_CHOICES = "MULTIPLE_CHOICES", _("Choix multiple")


class Question(models.Model):
    form = models.ForeignKey(
        to="eventsd_events.Form",
        related_name="questions",
        on_delete=models.CASCADE,
        verbose_name=_("Formulaire"),
    )
    text = models.CharField(
        verbose_name=_("Texte de la question"), max_length=1000
    )
    type = models.CharField(
        verbose_name=_("Type"),
        choices=QuestionType.choices,
        max_length=32,
    )
    mandatory = models.BooleanField(verbose_name=_("Obligatoire"))
    answers = models.TextField(
        verbose_name=_("Réponses possibles"),
        blank=True,
        null=True,
        max_length=500,
        help_text=_("Une réponse par ligne"),
    )
    order = models.PositiveSmallIntegerField(verbose_name=_("Ordre"))

    class Meta:
        verbose_name = _("question")
        verbose_name_plural = _("questions")
        ordering = ("order",)

    def clean(self):
        if (
            self.type
            in (
                QuestionType.CHOICE.value,
                QuestionType.MULTIPLE_CHOICES.value,
            )
            and (self.answers is None or self.answers == "")
        ):
            raise ValidationError(
                _(
                    "Pour une question de ce type, "
                    "vous devez préciser les choix possibles"
                )
            )

    def get_form_field_kwargs(self):
        return {
            "label": str(self),
            "label_suffix": None,
            "required": self.mandatory,
        }

    def get_form_field(self):
        if self.type == QuestionType.TEXT.value:
            return forms.fields.CharField(
                max_length=400, **self.get_form_field_kwargs()
            )

        if self.type == QuestionType.LONG_TEXT.value:
            return forms.fields.CharField(
                max_length=1_000,
                widget=forms.widgets.Textarea(),
                **self.get_form_field_kwargs(),
            )

        if self.type == QuestionType.CHOICE.value:
            return forms.fields.ChoiceField(
                choices=self.get_field_choices(),
                widget=forms.widgets.RadioSelect(),
                **self.get_form_field_kwargs(),
            )

        if self.type == QuestionType.MULTIPLE_CHOICES.value:
            return forms.fields.MultipleChoiceField(
                choices=self.get_field_choices(),
                widget=forms.widgets.CheckboxSelectMultiple(),
                **self.get_form_field_kwargs(),
            )

        raise NotImplementedError(
            "No form field implemented for this question type."
        )

    def get_field_tuple(self):
        return (str(self.id), self.get_form_field())

    def __str__(self):
        if self.mandatory:
            return self.text + " (*)"
        return self.text

    @property
    def possible_answers(self):
        return self.answers.splitlines()

    def get_field_choices(self):
        return [(answer, answer) for answer in self.possible_answers]


class FormAnswer(models.Model):
    attendee = models.ForeignKey(
        to="eventsd_events.Attendee",
        related_name="form_answers",
        verbose_name=_("Participant"),
        on_delete=models.CASCADE,
    )

    question = models.ForeignKey(
        to="eventsd_events.Question",
        related_name="form_answers",
        verbose_name=_("Question"),
        on_delete=models.CASCADE,
    )

    answer = models.TextField(
        verbose_name=_("Réponse"),
        max_length=10_000,
    )

    class Meta:
        verbose_name = _("réponse formulaire")
        verbose_name_plural = _("réponses formulaire")
        unique_together = (("attendee", "question"),)
        ordering = ("question__order",)

    def __str__(self):
        return self.answer


class AttendeeLabel(models.Model):
    title = models.CharField(max_length=120, verbose_name=_("Titre"))

    class Meta:
        verbose_name = _("label participant")
        verbose_name_plural = _("labels participant")
        ordering = ("title",)

    def __str__(self):
        return self.title



# Generated by Django 3.2.7 on 2021-09-22 18:31

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('eventsd_newsletter', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='address',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now, verbose_name="Date d'inscription"),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.7 on 2021-09-22 17:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eventsd_sponsors', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='sponsor',
            name='description',
            field=models.TextField(blank=True, max_length=2000, null=True, verbose_name='Description'),
        ),
        migrations.AddField(
            model_name='sponsor',
            name='featured',
            field=models.BooleanField(default=False, verbose_name='Mis en avant'),
        ),
        migrations.AddField(
            model_name='sponsor',
            name='website_url',
            field=models.URLField(blank=True, null=True, verbose_name='URL du site'),
        ),
    ]



from django.db import models
from django.contrib.auth.models import AbstractUser
from django.utils.translation import ugettext_lazy as _


class User(AbstractUser):
    address = models.TextField(
        max_length=1000,
        verbose_name=_("Adresse"),
        null=True,
        blank=True,
    )

    city = models.CharField(
        verbose_name=_("Ville"),
        max_length=150,
        null=True,
        blank=True,
    )

    zip_code = models.CharField(
        verbose_name=_("Code Postal"),
        max_length=20,
        null=True,
        blank=True,
    )

    country = models.CharField(
        verbose_name=_("Pays"),
        max_length=64,
        null=True,
        blank=True,
    )

    email = models.EmailField(
        null=False,
        blank=False,
        verbose_name=_("Adresse email"),
        unique=True,
    )

    def has_complete_address(self):
        return not any(
            f is None
            for f in (
                self.address,
                self.city,
                self.zip_code,
                self.country,
            )
        )



# Generated by Django 3.2.7 on 2021-09-13 09:41

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('eventsd_events', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='attendee',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Utilisateur'),
        ),
        migrations.AlterUniqueTogether(
            name='formanswer',
            unique_together={('attendee', 'question')},
        ),
    ]



from django.db import models
import hashlib
import uuid
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

def get_hash(token):
    m = hashlib.sha512()
    m.update(token.encode('utf-8'))
    return m.hexdigest()[:72]

class Address(models.Model):
    id = models.UUIDField(
        verbose_name=_("ID"),
        default=uuid.uuid4,
        editable=False,
        primary_key=True,
        unique=True,
    )

    email = models.EmailField(
        verbose_name=_("Adresse électronique")
    )

    confirmed = models.BooleanField(
        verbose_name=_("Confirmé"),
        default=False,
        editable=True,
    )

    created_at = models.DateTimeField(
        verbose_name=_("Date d'inscription"),
        auto_now_add=True,
    )

    @property
    def confirm_token(self):
        return get_hash(
            self.email +
            '_subscribe_' +
            settings.EVENTSD_NEWSLETTER_CONFIRM_SALT
        )
    
    @property
    def unsubscribe_token(self):
        return get_hash(
            self.email +
            '_unsubscribe_' +
            settings.EVENTSD_NEWSLETTER_CONFIRM_SALT
        )

    def __str__(self):
        return self.email
    
    class Meta:
        verbose_name = _("addresse")
        verbose_name_plural = _("adresses")


from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class SponsorsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "eventsd.sponsors"
    label = "eventsd_sponsors"
    verbose_name = _("Sponsors")



from django.db import models
from django.utils.translation import ugettext_lazy as _


class Sponsor(models.Model):
    name = models.CharField(
        verbose_name=_("Nom"),
        max_length=256,
    )

    description = models.TextField(
        verbose_name=_("Description"),
        max_length=2_000,
        blank=True,
        null=True,
    )

    website_url = models.URLField(
        verbose_name=_("URL du site"),
        blank=True,
        null=True,
    )

    featured = models.BooleanField(
        verbose_name=_("Mis en avant"),
        default=False,
        editable=True,
    )

    logo = models.FileField(
        verbose_name=_("Logo"),
    )

    enabled = models.BooleanField(
        verbose_name=_("Activé"),
    )

    order = models.PositiveIntegerField(
        verbose_name=_("Ordre"),
        default=1,
    )

    class Meta:
        ordering = ("order",)

    def __str__(self):
        return self.name



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from adminsortable2.admin import SortableInlineAdminMixin
from . import models


@admin.register(models.Center)
class CenterAdmin(admin.ModelAdmin):
    list_display = ("name",)
    search_fields = ("name", "address")
    fieldsets = (
        (None, {"fields": ("name", "address")}),
        (_("Coordonnées GPS"), {"fields": ("lat", "lng")}),
        (_("Notes"), {"fields": ("private_notes",)}),
    )


class EventDocumentInlineAdmin(admin.TabularInline):
    model = models.Event.documents.through
    fields = ("document", "display_name", "visibility")
    extra = 0


@admin.register(models.Event)
class EventAdmin(admin.ModelAdmin):
    list_display = ("name", "center", "start_date", "end_date")
    list_filter = ("center",)
    ordering = ("-start_date",)
    inlines = (EventDocumentInlineAdmin,)

    fieldsets = (
        (None, {"fields": ("name", "center")}),
        (
            _("Dates de l'évènement"),
            {
                "fields": ("start_date", "end_date"),
            },
        ),
        (
            _("Dates d'inscription"),
            {"fields": ("signup_start_date", "signup_end_date")},
        ),
        (
            _("Formulaires d'inscription"),
            {"fields": ("form",)},
        ),
        (
            _("Informations aux participants"),
            {"fields": ("description", "notes")},
        ),
    )


class QuestionInlineAdmin(SortableInlineAdminMixin, admin.TabularInline):
    model = models.Question
    fields = ("order", "text", "type", "mandatory", "answers")
    extra = 0


@admin.register(models.Form)
class FormAdmin(admin.ModelAdmin):
    search_fields = ("name",)
    inlines = (QuestionInlineAdmin,)


class FormAnswerInline(admin.TabularInline):
    model = models.FormAnswer
    fields = ("question", "answer")
    extra = 0


@admin.register(models.Attendee)
class AttendeeAdmin(admin.ModelAdmin):
    list_display = (
        "first_name",
        "last_name",
        "owner",
        "status",
        "event",
    )

    raw_id_fields = (
        "owner",
        "event",
    )

    search_fields = (
        "first_name",
        "last_name",
        "owner__username",
        "owner__first_name",
        "owner__last_name",
        "owner__email",
    )

    list_filter = (
        "status",
        "event",
        "labels",
    )

    filter_horizontal = ("labels",)

    fieldsets = (
        (None, {"fields": ("owner", "event")}),
        (
            _("Informations participant"),
            {
                "fields": (
                    "first_name",
                    "last_name",
                    "dob",
                )
            },
        ),
        (
            _("Sélection"),
            {
                "fields": ("status", "labels"),
            },
        ),
    )

    inlines = (FormAnswerInline,)


@admin.register(models.AttendeeLabel)
class AttendeeLabelAdmin(admin.ModelAdmin):
    search_fields = ("title",)


@admin.register(models.Document)
class DocumentAdmin(admin.ModelAdmin):
    search_fields = ("display_name", "admin_name")



from rest_framework import viewsets
from . import models, serializers


class SponsorViewset(viewsets.ReadOnlyModelViewSet):
    serializer_class = serializers.SponsorSerializer

    def get_queryset(self):
        return models.Sponsor.objects.filter(enabled=True)



from rest_framework import serializers
from . import models


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.User

        fields = (
            "username",
            "first_name",
            "last_name",
            "email",
            "address",
            "city",
            "zip_code",
            "country",
            "is_staff",
        )

        read_only_fields = (
            "first_name",
            "last_name",
            "email",
            "username",
            "email",
            "is_staff",
        )



from rest_framework import (
    viewsets,
    permissions,
    response,
)

from rest_framework.decorators import action

from .. import (
    serializers,
    models,
)


class EventViewset(viewsets.ReadOnlyModelViewSet):
    permission_classes = [permissions.AllowAny]
    serializer_class = serializers.EventSerializer
    queryset = models.Event.objects.get_visible_events()

    @action(detail=False, methods=["get"], url_path="open")
    def open_events(self, request):
        objects = models.Event.objects.get_open_events()
        serializer = self.get_serializer_class()
        return response.Response(serializer(objects, many=True).data)



from django.contrib import admin
from . import models

@admin.register(models.Address)
class AddressAdmin(admin.ModelAdmin):
    list_display = ('email', 'confirmed', 'created_at')
    list_filter = ('confirmed', )
    search_fields = ('email',)
    readonly_fields = ("confirm_token", "unsubscribe_token")


from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.validators import ValidationError
from django.conf import settings
from django.core.files.storage import default_storage
from pathlib import Path
from django.utils import timezone
from .signup import SelectionStatus


class Center(models.Model):
    name = models.CharField(verbose_name=_("Nom"), max_length=120)
    address = models.TextField(verbose_name=_("Adresse"), max_length=1000)
    lng = models.DecimalField(
        verbose_name=_("Longitude"),
        max_digits=9,
        decimal_places=6,
        null=True,
        blank=True,
    )
    lat = models.DecimalField(
        verbose_name=_("Latitude"),
        max_digits=9,
        decimal_places=6,
        null=True,
        blank=True,
    )
    private_notes = models.TextField(
        verbose_name=("Notes privées"), max_length=2000, blank=True, null=True
    )

    class Meta:
        verbose_name = _("centre")
        verbose_name_plural = _("centres")

    def __str__(self):
        return self.name


class EventManager(models.Manager):
    def get_open_events(self):
        return self.filter(
            signup_start_date__lte=timezone.now(),
            signup_end_date__gt=timezone.now(),
        )

    def get_visible_events(self):
        return self.filter(
            signup_start_date__lte=timezone.now(),
        )


class Event(models.Model):
    name = models.CharField(
        max_length=256,
        verbose_name=_("Nom de l'évènement"),
    )

    center = models.ForeignKey(
        to="eventsd_events.Center",
        verbose_name=_("Centre"),
        on_delete=models.CASCADE,
    )

    signup_start_date = models.DateTimeField(
        verbose_name=_("Date de début d'inscription")
    )
    signup_end_date = models.DateTimeField(
        verbose_name=_("Date de fin d'inscription")
    )

    start_date = models.DateTimeField(verbose_name=_("Date de début"))
    end_date = models.DateTimeField(verbose_name=_("Date de fin"))

    form = models.ForeignKey(
        to="eventsd_events.Form",
        verbose_name=_("Formulaire"),
        on_delete=models.SET_DEFAULT,
        default=1,
    )

    documents = models.ManyToManyField(
        to="eventsd_events.Document",
        related_name="events",
        verbose_name=_("Documents"),
        through="eventsd_events.EventDocument",
        blank=True,
    )

    notes = models.TextField(
        verbose_name=_("Notes"),
        help_text=_(
            "Ce texte, si défini est affiché "
            "lors de l'inscription à l'évènement "
            "(il peut éventuellement contenir des liens)"
        ),
        blank=True,
        null=True,
    )

    description = models.TextField(
        verbose_name=_("Description de l'évènement"),
        help_text=_(
            "Si ce champ n'est pas spécifié, "
            "la description du projet sera utilisée."
        ),
    )

    objects = EventManager()

    class Meta:
        verbose_name = _("évènement")
        verbose_name_plural = _("évènements")

    def clean(self):
        errors = {}

        if self.signup_start_date > self.signup_end_date:
            errors["signup_start_date"] = _(
                "La date début d'inscription doit être avant la date de "
                "fin d'inscription."
            )

        if self.signup_end_date > self.start_date:
            errors["signup_end_date"] = _(
                "La date de fin d'inscription doit être avant "
                "la date de début d'évènement."
            )

        if self.start_date > self.end_date:
            errors["end_date"] = _(
                "La date de fin d'évènement doit être après "
                "la date de début d'évènement."
            )

        if errors:
            raise ValidationError(errors)

        return self.form.questions.all()

    def __str__(self):
        return self.name

    def get_my_documents(self, attendee):
        if attendee.status == SelectionStatus.CONFIRMED.value:
            return self.documents.all()
        if attendee.status == SelectionStatus.ACCEPTED.value:
            return self.documents.filter(
                eventdocument__visibility__in=(
                    DocumentType.ACCEPTED_OR_CONFIRMED.value,
                    DocumentType.PUBLIC,
                )
            )
        else:
            return self.documents.filter(
                eventdocument__visibility=DocumentType.PUBLIC.value
            )


class DocumentType(models.IntegerChoices):
    PUBLIC = 1, _("Public")
    ACCEPTED_OR_CONFIRMED = 2, _("Accepté ou confirmé")
    CONFIRMED = 3, _("Confirmé seulement")


class EventDocument(models.Model):
    document = models.ForeignKey(
        to="eventsd_events.Document",
        verbose_name=_("Document"),
        on_delete=models.CASCADE,
    )

    event = models.ForeignKey(
        to="eventsd_events.Event",
        verbose_name=_("Évènement"),
        on_delete=models.CASCADE,
    )

    display_name = models.CharField(
        max_length=260,
        verbose_name=_("Nom public du document"),
        help_text=_(
            "Le nom public du document qui sera affiché aux participants"
        ),
    )

    visibility = models.IntegerField(
        verbose_name=_("Visibilité"),
        choices=DocumentType.choices,
    )

    class Meta:
        verbose_name = _("document lié à l'évènement")
        verbose_name_plural = _("documents lié à l'évènement")

    def __str__(self):
        return self.display_name


class Document(models.Model):
    admin_name = models.CharField(
        max_length=260,
        unique=True,
        verbose_name=_("Nom du document"),
        help_text=_("Ce nom est uniquement utilisé pour l'admin django"),
    )

    file = models.FileField(
        verbose_name=_("Fichier"),
        storage=getattr(settings, "EVENTSD_STORAGE_BACKEND", default_storage),
        upload_to=Path("eventsd") / "documents",
    )

    class Meta:
        verbose_name = _("document")
        verbose_name_plural = _("documents")

    def __str__(self):
        return self.admin_name
